@@ -12,6 +12,7 @@ class UndoableObject:
     it can supply get_reapply_action_args, which will allow the action to be "reapplied" to new
     arguments (for example, if the action is setting a text attribute, reapply might set that attribute
     for the currently highlighted text)."""
+    
     def __init__ (self, action, inverse, history,
                   action_args=None,
                   undo_action_args=None,
@@ -19,18 +20,24 @@ class UndoableObject:
                   get_reundo_action_args=None,
                   reapply_name=None,
                   reundo_name=None,
-                  is_undo=False):
-        if not action_args: action_args=[]
-        if not undo_action_args: undo_action_args=[]
-        self.history=history
-        self.action=action
-        self.inverse_action=inverse
+                  is_undo=False,
+                  widget=None, # Keep track of widget where this
+                               # action began (useful for GUIs to
+                               # check if actions have changed value
+                               # from "saved" state)
+                  ):
+        if not action_args: action_args = []
+        if not undo_action_args: undo_action_args = []
+        self.history = history
+        self.action = action
+        self.inverse_action = inverse
         self.get_reapply_action_args = get_reapply_action_args
         self.undo_action_args = undo_action_args
-        self.get_reundo_action_args=get_reundo_action_args
+        self.get_reundo_action_args = get_reundo_action_args
         self.reapply_name = reapply_name
         self.reundo_name = reundo_name
-        self.is_undo=is_undo
+        self.is_undo = is_undo  # If our action itself is an undo
+        self.widget = widget
         if self.get_reapply_action_args:
             self.reapplyable = True
         else:
@@ -41,13 +48,14 @@ class UndoableObject:
         self.action(*self.action_args)
         self.history.append(self)
 
-    def inverse (self):        
+    def inverse (self):
         u = UndoableObject(self.inverse_action, self.action, self.history,
                            action_args=self.undo_action_args,
                            undo_action_args=self.action_args,
                            get_reapply_action_args=self.get_reundo_action_args,
                            get_reundo_action_args=self.get_reapply_action_args,
-                           is_undo=not self.is_undo)
+                           is_undo=not self.is_undo,
+                           widget=self.widget)
         self.history.remove(self)
         u.perform()
 
@@ -61,18 +69,23 @@ class UndoableObject:
                                reapply_name=self.reundo_name, reundo_name=self.reapply_name)
             u.perform()
 
+    def __repr__ (self):
+        return '<Undo.UndoableObject %s widget=%s>'%(self.is_undo,self.widget)
+
 class UndoableTextChange (UndoableObject):
-    def __init__ (self, set_text_action, history, initial_text="",text="",txt_id=None):
+    def __init__ (self, set_text_action, history, initial_text="",text="",txt_id=None,is_undo=False):
         self.txt_id = txt_id
         self.blob_matcher = re.compile('\s+\S+\s+')
         self.initial_text = initial_text
         self.text = text
         self._set_text = set_text_action
-        UndoableObject.__init__(self,lambda *args: self._set_text(self.text),lambda *args: self._set_text(self.initial_text),history)
+        UndoableObject.__init__(self,lambda *args: self._set_text(self.text),lambda *args: self._set_text(self.initial_text),history,
+                                is_undo=is_undo,widget=self.txt_id)
         self.mode=self.determine_mode()
         try:
             self.cindex,self.clen = self.find_change(self.text)
         except TooManyChanges:
+            debug('Too many changes - assume 0,0',0)
             self.cindex,self.clen = 0,0
 
     def determine_mode (self,text=None,initial_text=None):
@@ -83,21 +96,18 @@ class UndoableTextChange (UndoableObject):
         elif len(text) < len(initial_text):
             return 'delete'
 
-    def find_change (self, text2=None):
+    def find_change (self, text2=None, initial_text=None):
+        if initial_text is None: initial_text = self.initial_text
         if not self.mode:
             self.text = text2
             self.determine_mode()
-        if not text2: text2=self.text
-        if not hasattr(self,'sm'):
-            self.sm = difflib.SequenceMatcher(None,self.initial_text,text2)
-        else:
-            self.sm.set_seq2(text2)
-        blocks=self.sm.get_matching_blocks()
+        if text2 is None: text2=self.text
+        blocks = difflib.SequenceMatcher(None,initial_text,text2).get_matching_blocks()
         # we only are interested in similar blocks at different positions
         # (which tell us where the changes happened).
         ch_blocks = filter(lambda x: x[0] != x[1] and x[2] != 0, blocks)
         if ch_blocks and len(ch_blocks)>1:
-            raise TooManyChanges("More than one block changed: %s in %s"%(ch_blocks,text2))
+            raise TooManyChanges("More than one block changed from '%s' to '%s': %s"%(initial_text,text2,ch_blocks))
         if ch_blocks:
             i,j,n = ch_blocks[0]
             change_length = j-i
@@ -105,45 +115,96 @@ class UndoableTextChange (UndoableObject):
             return [change_index,change_length]
         else:
             if self.mode=='delete':
-                return [len(self.initial_text),len(self.initial_text)-len(text2)]
+                return [len(initial_text),len(initial_text)-len(text2)]
             else: #self.mode=='add', we presume
-                return [len(self.initial_text),len(text2)-len(self.initial_text)]
+                return [len(initial_text),len(text2)-len(initial_text)]
 
     def add_text (self, new_text):
-        mode=self.determine_mode(new_text)
+        mode=self.determine_mode(new_text,self.text)
         contmode = self.determine_mode(new_text,self.text)
         if (mode == contmode == self.mode):
             try:
+                # This whole thing is in a try/except box because
+                # there are multiple places where "Too many changes"
+                # could crop up as an error.
                 cindex,clen = self.find_change(new_text)
-            except TooManyChanges:
-                self.new_action(new_text)
-            else:
                 if ((cindex==self.cindex) or
-                    (self.mode=='delete' and cindex==self.cindex-(clen-self.clen))
+                    (self.mode=='add' and cindex==self.cindex) or
+                    (self.mode=='delete' and cindex==(self.cindex-clen))
                     ):
-                    changed_text = new_text[cindex:cindex+clen]
-                    if not self.blob_matcher.search(changed_text):
+                    if self.mode=='add': changed_text = new_text[cindex:cindex+(self.clen+clen)]
+                    else: changed_text=''
+                    # Now we make sure the addition is at the end or middle of our new text...
+                    relative_cindex,relative_clen = self.find_change(new_text,self.text)
+                    # We only consider a word at a time to be a continuous change...
+                    if (
+                        # Adding text to the end
+                        (
+                        relative_cindex==cindex+(clen-relative_clen) 
+                        or
+                        # Adding text to the beginning
+                        (relative_cindex==self.clen or relative_cindex==self.cindex) 
+                        )
+                        and
+                        not self.blob_matcher.search(changed_text)
+                        ):
                         self.text = new_text
-                        self.cindex,self.clen = cindex,clen
-                        return
+                        self.cindex,self.clen = self.find_change(new_text)
+                        ## End here if we've worked
+                        return                
+            except TooManyChanges:
+                pass # We'll go to the end of the method & create a new action
+        # If the mode has changed or we have too many changes to
+        # handle simply, we just create a new undo action for this new
+        # change
         self.new_action(new_text)
 
     def new_action (self, new_text):
         self.history.append(UndoableTextChange(self._set_text,self.history,
-                                                   initial_text=self.text, text=new_text)
+                                               initial_text=self.text, text=new_text,
+                                               txt_id=self.txt_id,
+                                               )
                             )
 
     def inverse (self):
-        self._set_text(self.initial_text)
+        #self._set_text(self.initial_text)
         u=UndoableTextChange(self._set_text,self.history,initial_text=self.text,
-                             text=self.initial_text)
-        u.is_undo=not self.is_undo
+                             text=self.initial_text,txt_id=self.txt_id,
+                             is_undo=not self.is_undo,
+                             )
         self.history.remove(self)
         u.perform()
 
     def perform (self):
+        self._set_text(self.text)
         self.history.append(self)
 
+    def __repr__ (self):
+        return '<Undo.UndoableTextChange '+repr(self.mode)+' '+repr(self.txt_id)+\
+               repr(self.initial_text)+'=>'+repr(self.text)+'clen'+str(self.clen)+\
+               'cindex'+str(self.cindex)+'>'
+
+    def __eq__ (self,obj):
+        """We are equal too objects that represent the same txt_id and
+        are either the same action as us or the inverse of us
+        """
+        if not hasattr(obj,'txt_id'):
+            return False
+        else:
+            return (
+                self.txt_id==obj.txt_id
+                and
+                ((self.initial_text==obj.initial_text
+                  and
+                  self.text == obj.text)
+                 or
+                 (self.initial_text==obj.text
+                  and
+                  self.text==obj.initial_text)
+                 )
+                )
+    
+
 class UndoableTextContainer:
     def __init__ (self, container, history):
         self.history = history
@@ -157,8 +218,8 @@ class UndoableTextContainer:
         if self._setting:
             return
         txt = self.get_text()
-        if txt == self.txt: pass
-        if len(self.history)>1 and hasattr(self.history[-1],'txt_id') and self.history[-1].txt_id==self.container:
+        if txt == self.txt: return
+        if len(self.history)>=1 and hasattr(self.history[-1],'txt_id') and self.history[-1].txt_id==self.container:
             self.change = self.history[-1]
             self.history[-1].add_text(txt)
         else:
@@ -174,11 +235,33 @@ class UndoableTextContainer:
     def get_text (self): raise NotImplementedError
 
     def _set_text (self,txt):
-        self._setting = True
-        self.set_text(txt)
-        self._setting = False
-        
-    def set_text (self,txt): raise NotImplementedError
+        self._setting = True # Set flag so we don't trigger our own
+                             # callbacks
+        orig = self.get_text() # Get current text for comparison
+                               # (helps with placement of cursor)
+        if len(txt) > orig:         # If we're adding
+            try:
+                index,length = self.change.find_change(txt,orig)
+                cursor_index = index+length
+            except TooManyChanges:
+                # If we changed more than one block, there is no obvious
+                # place to put the cursor, so we put it at the end
+                cursor_index = len(txt)
+        else: # If we're Removing
+            try:
+                # We get the *index* of the inverse of our action
+                # i.e. If we go from Thhis -> This, then we reverse to
+                # This -> Thhis (which is the same as where it would
+                # have been after hitting backspace) and we put it
+                # there.
+                index,length = self.change.find_change(orig,txt)
+                cursor_index = index
+            except TooManyChanges:
+                cursor_index = len(txt)
+        self.set_text(txt,cursor_index)
+        self._setting = False # Unset flag
+
+    def set_text (self,txt,cursor_index): raise NotImplementedError
 
 class UndoableEntry (UndoableTextContainer):
     def __init__ (self, entry,history):
@@ -191,33 +274,52 @@ class UndoableEntry (UndoableTextContainer):
                            self.change_event_cb
                            )
         
-    def set_text (self, txt):
-        index,length = self.change.find_change(txt)
-        self.entry.set_text(txt)
+    def set_text (self, txt, cursor_index):
         self.entry.grab_focus()
-        self.entry.set_position(index + length)
+        self.entry.set_text(txt)
+        self.entry.set_position(cursor_index)
         
 class UndoableGenericWidget:
+    """Wrap a widget in an Undo class.
+
+    We take a set_method, a get_method, and a signal, as well as a
+    history list to append our Undo objects to.
+    """
     def __init__ (self, widget, history, set_method='set_value',get_method='get_value', signal='changed'):
         self.w = widget
         self.set_method = set_method
         self.get_method = get_method
-        self.set = getattr(self.w,self.set_method)
-        self.get = getattr(self.w,self.get_method)
+        self.im_doing_the_setting = False
         self.history = history
         self.last_value = self.get()
         self.w.connect(signal,self.changecb)
 
+    def set (self, val):
+        self.im_doing_the_setting = True
+        getattr(self.w,self.set_method)(val)
+        self.im_doing_the_setting = False
+
+    def get (self):
+        return getattr(self.w,self.get_method)()
+
     def changecb (self,*args):
+        if self.im_doing_the_setting:
+            # If we are doing the setting, presumably this is from one
+            # of the Undo objects we created, in which case we don't
+            # want to add a new item to the Undo queue
+            return
         old_val = self.last_value
         new_val = self.get()
+        if old_val==new_val: return # Ignore redundant changes...
         if new_val != old_val:
             # We don't perform because we're being called after the action has happened.
             # We simply append ourselves to the history list.
-            self.history.append(UndoableObject(lambda *args: self.set(new_val),
-                                               lambda *args: self.set(old_val),
-                                               self.history)
-                                )
+            u = UndoableObject(lambda *args: self.set(new_val),
+                               lambda *args: self.set(old_val),
+                               self.history,
+                               widget=self.w
+                               )
+            self.history.append(u)
             self.last_value=new_val
         
 class UndoableTextView (UndoableTextContainer):
@@ -232,14 +334,10 @@ class UndoableTextView (UndoableTextContainer):
         self.buffer.connect('apply-tag',self.change_event_cb)
         self.buffer.connect('remove-tag',self.change_event_cb)
         
-    def set_text (self, text):        
+    def set_text (self, text, cursor_index):        
         self.buffer.set_text(text)
         self.tv.grab_focus()
-        try:
-            index,length = self.change.find_change(text)
-            self.buffer.place_cursor(self.buffer.get_iter_at_offset(index + length))
-        except TooManyChanges:
-            print 'WARNING: THAT SHOULDNT HAVE HAPPENED! Too many changes!'
+        self.buffer.place_cursor(self.buffer.get_iter_at_offset(cursor_index))
 
     def get_text (self):
         return self.buffer.get_text(self.buffer.get_start_iter(),
@@ -251,6 +349,7 @@ class UndoHistoryList (list):
         self.undo_widget = undo_widget
         self.redo_widget = redo_widget
         self.reapply_widget = reapply_widget
+        self.action_hooks = []
         list.__init__(self)
         self.gui_update()
         if signal:
@@ -258,16 +357,32 @@ class UndoHistoryList (list):
             if self.redo_widget: self.redo_widget.connect(signal,self.redo)
             if self.reapply_widget: self.reapply_widget.connect(signal,self.reapply)
 
+    def add_action_hook (self, hook):
+        """Add action hook
+
+        Hook will be a callback as follows
+
+        hook(undo_history,action,type)
+
+        where undo_history is this instance of UndoHistory and
+        type is 'perform','undo','redo', or 'reapply'
+        """
+        self.action_hooks.append(hook)
+
     def undo (self, *args):
         index = -1
         if len(self) == 0: return False
         try:
             while self[index].is_undo:
-                index = index - 1
-        except:
+                index -= 1
+        except IndexError:
             debug('All %s available action are .is_undo=True'%len(self),0)
+            print 'There is nothing to undo!'
+            print 'All %s actions are undos'%len(self)
+            print self,index
             raise
         self[index].inverse()
+        for h in self.action_hooks: h(self,self[index],'undo')
 
     def redo (self, *args):
         if len(self) == 0: return False
@@ -275,14 +390,18 @@ class UndoHistoryList (list):
         try:
             while not self[index].is_undo:
                 index = index - 1
-        except:
+        except IndexError:
             debug('All %s available actions are is_undo=False'%len(self),0)
+            print 'There is nothing to redo!'
+            print 'All %s available actions are is_undo=False'%len(self)
             raise
         self[index].inverse()
+        for h in self.action_hooks: h(self,self[index],'redo')
 
     def reapply (self, *args):
         debug('Reapplying',0)
         self[-1].reapply()
+        for h in self.action_hooks: h(self,self[-1],'reapply')
 
     def set_sensitive (self,w,val):
         debug('set_sensitive',0)
@@ -331,6 +450,9 @@ class UndoHistoryList (list):
     def append (self,obj):
         debug('Appending %s'%obj,0)
         list.append(self,obj)
+        if obj.is_undo==False: # Is this necessary? Not sure...
+            for h in self.action_hooks:
+                h(self,obj,'perform')
         self.gui_update()
 
     def remove (self,obj):
@@ -350,6 +472,7 @@ class MultipleUndoLists:
         self.signal = signal
         self.get_current_id = get_current_id
         self.histories = {}
+        self.action_hooks = []
         if signal:
             if self.undo_widget: self.undo_widget.connect(signal,self.undo)
             if self.redo_widget: self.redo_widget.connect(signal,self.redo)
@@ -358,6 +481,9 @@ class MultipleUndoLists:
         # attempts to implement the following programatically are failing me...
         # it feels awful to write each of these methods out here, but here goes...
 
+    def add_action_hook (self, hook):
+        self.action_hooks.append(hook)
+        for h in self.get_all_histories(): h.add_action_hook(hook)
     def __add__ (self,*args,**kwargs): return self.get_history().__add__(*args,**kwargs)
     def __contains__ (self,*args,**kwargs): return self.get_history().__contains__(*args,**kwargs)
     def __delitem__ (self,*args,**kwargs): return self.get_history().__delitem__(*args,**kwargs)
@@ -394,10 +520,17 @@ class MultipleUndoLists:
     def remove (self,*args,**kwargs): return self.get_history().remove(*args,**kwargs)
     def reverse (self,*args,**kwargs): return self.get_history().reverse(*args,**kwargs)
     def sort (self,*args,**kwargs): return self.get_history().sort(*args,**kwargs)
-    def redo (self,*args,**kwargs): return self.get_history().redo(*args,**kwargs)
-    def undo (self,*args,**kwargs): return self.get_history().undo(*args,**kwargs)
+
+    def redo (self,*args,**kwargs):
+        return self.get_history().redo(*args,**kwargs)
+
+    def undo (self,*args,**kwargs):
+        return self.get_history().undo(*args,**kwargs)
+    
     def reapply (self,*args,**kwargs): return self.get_history().reapply(*args,**kwargs)
 
+    def get_all_histories (self): return self.histories.values()
+
     def get_history (self):
         hid=self.get_current_id()
         if self.histories.has_key(hid):
@@ -409,7 +542,9 @@ class MultipleUndoLists:
             return self.histories[hid]
 
     def make_history (self):
-        return UndoHistoryList(self.undo_widget,self.redo_widget,None,None)
+        uhl = UndoHistoryList(self.undo_widget,self.redo_widget,None,None)
+        for h in self.action_hooks: uhl.add_action_hook(h)
+        return uhl
 
     def switch_context (self, hid):
         # set sensitivity for current context
@@ -427,6 +562,12 @@ if __name__ == '__main__':
     import gtk
     w = gtk.Window()
     e = gtk.Entry()
+    sb = gtk.SpinButton()
+    adj = sb.get_adjustment()
+    adj.upper=100
+    adj.lower=-100
+    adj.step_increment = 1
+    adj.page_increment = 10
     tv = gtk.TextView()
     ub = gtk.Button(stock=gtk.STOCK_UNDO)
     rb = gtk.Button(stock=gtk.STOCK_REDO)
@@ -439,15 +580,20 @@ if __name__ == '__main__':
     vb.add(bb)
     vb.add(e)
     vb.add(tv)
+    vb.add(sb); sb.show()
     w.add(vb)
     uhl = UndoHistoryList(ub,rb,signal='clicked')
     UndoableTextView(tv,uhl)
     UndoableEntry(e,uhl)
+    UndoableGenericWidget(sb,uhl)
     w.show_all()
     w.connect('delete-event',lambda *args:gtk.main_quit())
     def show_changes (*args):
         for c in uhl:
-            print c,' initial: ',c.initial_text,' current: ',c.text
+            if hasattr(c,'initial_text'):
+                print c,' initial: ',c.initial_text,' current: ',c.text
+            else:
+                print c
     ub.connect('clicked',lambda *args: debug('Undo clicked!',0))
     sc.connect('clicked',show_changes)
     rb.connect('clicked',lambda *args: debug('Redo clicked!',0))    
