@@ -506,3 +506,28 @@ VOL_TO_MASS_TABLE = {
     ("l", "kg") : 1}
     
 
+# These functions are rather important! Our goal is simply to
+# facilitate look ups -- if the user types in "tomatoes", we want to
+# find "tomato." Note that the strings these functions produce will
+# _never_ be shown to the user, so it's fine to generate nonsense
+# words as well as correct answers -- our goal is to generate a list
+# of possible hits rather than to get the plural/singular form "right".
+
+# A quick look at the wikipedia suggests that plural forms of nouns
+# may be rather complex
+# http://en.wikipedia.org/wiki/Slovak_declension#Nouns so I won't
+# attempt it myself.
+
+# If someone implements this at a later date, please note that if
+# guess_singulars() returns something, guess_plurals() usually won't
+# be called (Gourmet uses these when trying to find a key for a word
+# -- if it sees "Tomatoes" and believes that the word is "Tomato", it
+# won't bother trying to guess "Tomatoeses".
+
+def guess_singulars (s): return []
+def guess_plurals (s): return []
+
+IGNORE=[]
+
+NUMBERS = {
+    }
