@@ -1,4 +1,4 @@
-import re, Image, os.path, os, xml.sax.saxutils, time, shutil, urllib, textwrap
+import re, Image, os.path, os, xml.sax.saxutils, time, shutil, urllib, textwrap, types
 from gourmet import gglobals, convert
 from gourmet.gdebug import *
 from gettext import gettext as _
@@ -101,6 +101,7 @@ class exporter:
         self.write_attr_foot()
 
     def _write_text_ (self):
+        #print 'exporter._write_text_',self.text_attr_order,'!'
         for a in self.text_attr_order:
             # This code will never be called for Gourmet
             # proper... here for convenience of symbiotic project...
@@ -113,13 +114,14 @@ class exporter:
                         s = dct.get('text','')
                         img = dct.get('image','')
                         time = dct.get('time',0)
-                        print 'Exporter sees step AS:'
-                        print '  text:',s
-                        print '  image:',img
-                        print '  time:',time
+                        #print 'Exporter sees step AS:'
+                        #print '  text:',s
+                        #print '  image:',img
+                        #print '  time:',time
                     else:
                         img = ''
-                    if self.do_markup: txt=self.handle_markup(s)
+                    if self.do_markup:
+                        txt=self.handle_markup(s)
                     if not self.use_ml: txt = xml.sax.saxutils.unescape(s)
                     if self.convert_attnames:
                         out_a = gglobals.TEXT_ATTR_DIC[a]
@@ -140,7 +142,8 @@ class exporter:
             # End of non-Gourmet code
             txt=self._grab_attr_(self.r,a)
             if txt and txt.strip():
-                if self.do_markup: txt=self.handle_markup(txt)
+                if self.do_markup:  txt=self.handle_markup(txt)
+                #else: print 'exporter: do_markup=False'
                 if not self.use_ml: txt = xml.sax.saxutils.unescape(txt)
                 if self.convert_attnames:
                     self.write_text(gglobals.TEXT_ATTR_DIC[a],txt)
@@ -202,13 +205,12 @@ class exporter:
                 else:
                     ret = "%s/5 %s"%(ret/2.0,_('stars'))
             elif attr=='servings' and type(ret)!=str:
-                print 'USING',self.fractions
                 ret = convert.float_to_frac(ret,fractions=self.fractions)
             if type(ret) in [str,unicode] and attr not in ['thumb','image']:
                 try:
                     ret = ret.encode(self.DEFAULT_ENCODING)
                 except:
-                    print "wtf:",ret,"doesn't look like unicode."
+                    print "oops:",ret,"doesn't look like unicode."
                     raise
             return ret
 
@@ -288,8 +290,15 @@ class exporter:
         while more:
             fd,lang,atts=ai.get_font()
             chunk = xml.sax.saxutils.escape(txt.__getslice__(*ai.range()))
+            trailing_newline = ''
             fields=fd.get_set_fields()
             if fields != 0: #if there are fields
+                # Sometimes we get trailing newlines, which is ugly
+                # because we end up with e.g. <b>Foo\n</b>
+                #
+                # So we define trailing_newline as a variable
+                if chunk and chunk[-1]=='\n':
+                    trailing_newline = '\n'; chunk = chunk[:-1]
                 if 'style' in fields.value_nicks and fd.get_style()==pango.STYLE_ITALIC:
                     chunk=self.handle_italic(chunk)
                 if 'weight' in fields.value_nicks and fd.get_weight()==pango.WEIGHT_BOLD:
@@ -297,7 +306,7 @@ class exporter:
             for att in atts:
                 if att.type==pango.ATTR_UNDERLINE and att.value==pango.UNDERLINE_SINGLE:
                     chunk=self.handle_underline(chunk)
-            outtxt += chunk
+            outtxt += chunk + trailing_newline
             more=ai.next()
         return outtxt
 
@@ -390,7 +399,12 @@ class exporter_mult (exporter):
         """        
         if attr=='servings' and self.mult:
             ret = getattr(obj,attr)
-            fl_ret = convert.frac_to_float(ret)
+            if type(ret) in [int,float]:
+                fl_ret = float(ret)
+            else:
+                if ret is not None:
+                    print 'WARNING: IGNORING serving value ',ret
+                fl_ret = None
             if fl_ret:
                 return convert.float_to_frac(fl_ret * self.mult,
                                              fractions=self.fractions)
@@ -479,11 +493,11 @@ class ExporterMultirec:
                     ret = "%s/5 %s"%(ret/2,_('stars'))
                 else:
                     ret = "%s/5 %s"%(ret/2.0,_('stars'))
-            if type(ret) in [str,unicode] and attr not in ['thumb','image']:
+            if type(ret) in types.StringTypes and attr not in ['thumb','image']:
                 try:
                     ret = ret.encode(self.DEFAULT_ENCODING)
                 except:
-                    print "wtf:",ret,"doesn't look like unicode."
+                    print "oops:",ret,"doesn't look like unicode."
                     raise
             return ret
         
